package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.OutgoingContents;
import digital.laboratory.platform.reagent.service.OutgoingContentsService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (出库内容)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (出库内容) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/outgoing_contents" )
@Api(value = "outgoing_contents", tags = "(出库内容)管理")
public class OutgoingContentsController {

    private final  OutgoingContentsService outgoingContentsService;

	/**
	 * 通过id查询(出库内容)
	 * @param outgoingContentsId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{outgoingContentsId}" )
	@PreAuthorize("@pms.hasPermission('reagent_outgoing_contents_get')" )
	public R<OutgoingContents> getById(@PathVariable("outgoingContentsId" ) String outgoingContentsId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        OutgoingContents outgoingContents = outgoingContentsService.getById(outgoingContentsId);
		return R.ok(outgoingContents);
		//return R.ok(outgoingContentsService.getById(outgoingContentsId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param outgoingContents (出库内容)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_outgoing_contents_get')" )
    public R<IPage<OutgoingContents>> getOutgoingContentsPage(Page<OutgoingContents> page, OutgoingContents outgoingContents, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<OutgoingContents> outgoingContentsSList = outgoingContentsService.page(page, Wrappers.<OutgoingContents>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(outgoingContentsSList);
//		return R.ok(outgoingContentsService.page(page, Wrappers.query(outgoingContents)));
    }


    /**
     * 新增(出库内容)
     * @param outgoingContents (出库内容)
     * @return R
     */
    @ApiOperation(value = "新增(出库内容)", notes = "新增(出库内容)")
    @SysLog("新增(出库内容)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_outgoing_contents_add')" )
    public R<OutgoingContents> postAddObject(@RequestBody OutgoingContents outgoingContents, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        outgoingContents.setId(IdWorker.get32UUID().toUpperCase());
		if (outgoingContentsService.save(outgoingContents)) {
			return R.ok(outgoingContents, "对象创建成功");
		}
		else {
			return R.failed(outgoingContents, "对象创建失败");
		}
    }

    /**
     * 修改(出库内容)
     * @param outgoingContents (出库内容)
     * @return R
     */
    @ApiOperation(value = "修改(出库内容)", notes = "修改(出库内容)")
    @SysLog("修改(出库内容)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_outgoing_contents_edit')" )
    public R<OutgoingContents> putUpdateById(@RequestBody OutgoingContents outgoingContents, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (outgoingContentsService.updateById(outgoingContents)) {
			return R.ok(outgoingContents, "保存对象成功");
		}
		else {
			return R.failed(outgoingContents, "保存对象失败");
		}
    }

    /**
     * 通过id删除(出库内容)
     * @param outgoingContentsId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(出库内容)", notes = "通过id删除(出库内容)")
    @SysLog("通过id删除(出库内容)" )
    @DeleteMapping("/{outgoingContentsId}" )
    @PreAuthorize("@pms.hasPermission('reagent_outgoing_contents_del')" )
    public R<OutgoingContents> deleteById(@PathVariable String outgoingContentsId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        OutgoingContents oldOutgoingContents = outgoingContentsService.getById(outgoingContentsId);

		if (outgoingContentsService.removeById(outgoingContentsId)) {
			return R.ok(oldOutgoingContents, "对象删除成功");
		}
		else {
			return R.failed(oldOutgoingContents, "对象删除失败");
		}

    }

}
