package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.ProcurementContent;
import digital.laboratory.platform.reagent.service.ProcurementContentService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (采购内容)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购内容) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/procurement_content" )
@Api(value = "procurement_content", tags = "(采购内容)管理")
public class ProcurementContentController {

    private final  ProcurementContentService procurementContentService;

	/**
	 * 通过id查询(采购内容)
	 * @param procurementContentId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{procurementContentId}" )
	@PreAuthorize("@pms.hasPermission('reagent_procurement_content_get')" )
	public R<ProcurementContent> getById(@PathVariable("procurementContentId" ) String procurementContentId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ProcurementContent procurementContent = procurementContentService.getById(procurementContentId);
		return R.ok(procurementContent);
		//return R.ok(procurementContentService.getById(procurementContentId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param procurementContent (采购内容)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_procurement_content_get')" )
    public R<IPage<ProcurementContent>> getProcurementContentPage(Page<ProcurementContent> page, ProcurementContent procurementContent, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<ProcurementContent> procurementContentSList = procurementContentService.page(page, Wrappers.<ProcurementContent>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(procurementContentSList);
//		return R.ok(procurementContentService.page(page, Wrappers.query(procurementContent)));
    }


    /**
     * 新增(采购内容)
     * @param procurementContent (采购内容)
     * @return R
     */
    @ApiOperation(value = "新增(采购内容)", notes = "新增(采购内容)")
    @SysLog("新增(采购内容)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_procurement_content_add')" )
    public R<ProcurementContent> postAddObject(@RequestBody ProcurementContent procurementContent, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        procurementContent.setProcurementContentId(IdWorker.get32UUID().toUpperCase());
		if (procurementContentService.save(procurementContent)) {
			return R.ok(procurementContent, "对象创建成功");
		}
		else {
			return R.failed(procurementContent, "对象创建失败");
		}
    }

    /**
     * 修改(采购内容)
     * @param procurementContent (采购内容)
     * @return R
     */
    @ApiOperation(value = "修改(采购内容)", notes = "修改(采购内容)")
    @SysLog("修改(采购内容)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_procurement_content_edit')" )
    public R<ProcurementContent> putUpdateById(@RequestBody ProcurementContent procurementContent, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (procurementContentService.updateById(procurementContent)) {
			return R.ok(procurementContent, "保存对象成功");
		}
		else {
			return R.failed(procurementContent, "保存对象失败");
		}
    }

    /**
     * 通过id删除(采购内容)
     * @param procurementContentId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(采购内容)", notes = "通过id删除(采购内容)")
    @SysLog("通过id删除(采购内容)" )
    @DeleteMapping("/{procurementContentId}" )
    @PreAuthorize("@pms.hasPermission('reagent_procurement_content_del')" )
    public R<ProcurementContent> deleteById(@PathVariable String procurementContentId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ProcurementContent oldProcurementContent = procurementContentService.getById(procurementContentId);

		if (procurementContentService.removeById(procurementContentId)) {
			return R.ok(oldProcurementContent, "对象删除成功");
		}
		else {
			return R.failed(oldProcurementContent, "对象删除失败");
		}

    }

}
